package info.bukova.isspst.ui;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.ParseException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.zkoss.bind.BindContext;
import org.zkoss.bind.Converter;
import org.zkoss.zk.ui.Component;

public class BigDecimalConverter implements Converter<String, BigDecimal, Component>
{
	private final static Logger log = LoggerFactory.getLogger(BigDecimalConverter.class.getName());

	@Override
	public BigDecimal coerceToBean(String str, Component component, BindContext cx)
	{
		BigDecimal val = BigDecimal.ZERO;
		
		if (str != null)
		{
			try
			{
				DecimalFormat format = new DecimalFormat();
				format.setParseBigDecimal(true);
				val = (BigDecimal) format.parse(str);
			}
			catch (NumberFormatException e)
			{
				log.warn(str, e);
			}
			catch (ParseException e)
			{
				log.warn(str, e);
			}
		}
		
		return val;
	}

	@Override
	public String coerceToUi(BigDecimal val, Component component, BindContext cx)
	{
		if (val == null)
		{
			val = BigDecimal.ZERO;
		}

		val = val.setScale(2, BigDecimal.ROUND_DOWN);

		DecimalFormat format = new DecimalFormat();
		format.setMaximumFractionDigits(2);
		format.setMinimumFractionDigits(2);
		format.setGroupingUsed(true);
		format.setGroupingSize(3);

		return format.format(val);
	}

}
